///////////////////////////////////////////////////////////////
//  Serial communication
//
// FILE: comm.cpp
//
// CREATED:07.09.1999
// (C) Borisov E.S. , Kiev UA
///////////////////////////////////////////////////////////////
#ifndef STRICT
#define STRICT
#endif

#include "stdafx.h"
#include "Const.h"
#include "funclib.h"
#include "comm.h"

/////////////////////////////////////////////////////////////////
// FUNCTION : CSerialComm::CSerialComm();
// PURPOSE :   
// 
CSerialComm::CSerialComm():m_hPort(INVALID_HANDLE_VALUE), 
						 m_fConnect(FALSE), 
						 m_bPort(2),
						 m_dwBaudRate(CBR_9600),
						 m_bByteSize(8),
						 m_bParity(NOPARITY),
						 m_bStopBits(TWOSTOPBITS)
{
}
//////////////////////////////////////////////////////////////////
//  FUNCTION: CSerialComm::CSerialComm(...)
//  PURPOSE:  
CSerialComm::CSerialComm( BYTE  bPortA,
			  DWORD dwBaudRateA ,
			  BYTE  bByteSizeA,
			  BYTE  bParityA ,
			  BYTE  bStopBitsA ):
			  
			  m_hPort(INVALID_HANDLE_VALUE),
			  m_fConnect(FALSE),
			  m_bPort(bPortA),
			  m_dwBaudRate(dwBaudRateA),
			  m_bByteSize(bByteSizeA),
			  m_bParity(bParityA),
			  m_bStopBits(bStopBitsA)			  
{
} 
/////////////////////////////////////////////////////////////////
// FUNCTION : CSerialComm::~CSerialComm();
// PURPOSE : 
// 
CSerialComm::~CSerialComm()
{
	ClosePort();
}
/////////////////////////////////////////////////////////////////
// FUNCTION : VOID CSerialComm::OpenPort()
// PURPOSE :  
// 
VOID CSerialComm::OpenPort()
{
   CHAR   szPort[10];
   BOOL       fRetVal = FALSE;
   COMMTIMEOUTS ctoPort;

   m_fConnect = FALSE;
   sprintf(szPort,"COM%d", m_bPort) ;
   /////////////////////////////////////////////////////////////////
   //Create COMM
    m_hPort = CreateFile( szPort,
						GENERIC_READ | GENERIC_WRITE,
						0,
						0,
						OPEN_EXISTING,
						FILE_ATTRIBUTE_NORMAL,
						0 );
	if (m_hPort == INVALID_HANDLE_VALUE )
	{
 	    m_fConnect = FALSE;
		errorMsg(m_bPort,"CreateFile");
		return;
	}
   //
   /////////////////////////////////////////////////////////////////
   
   /////////////////////////////////////////////////////////////////
   //RX TX Queue
   	if(!SetupComm( m_hPort,INQUEUE,OUTQUEUE))
	{
		errorMsg(m_bPort,"SetupComm");
		ClosePort();
		return;
	}
   //
   /////////////////////////////////////////////////////////////////
   
   /////////////////////////////////////////////////////////////////
   //Clear Port
   	if(!PurgeComm( m_hPort,PURGE_TXABORT|PURGE_RXABORT|PURGE_TXCLEAR|PURGE_RXCLEAR))
	{
		errorMsg(m_bPort,"PurgeComm");
		ClosePort();
		return;
	}
   //
   /////////////////////////////////////////////////////////////////
	   
   /////////////////////////////////////////////////////////////////
   //Timeouts
	ctoPort.ReadIntervalTimeout = 0xFFFFFFFF ;
  	ctoPort.ReadTotalTimeoutMultiplier = 0 ;
  	ctoPort.ReadTotalTimeoutConstant = TIMEOUTS_READ_NORMAL ;
    //
	//no wiriting timeouts
	//
	ctoPort.WriteTotalTimeoutMultiplier = 0;// 2*CBR_9600/m_dwBaudRate ;
  	ctoPort.WriteTotalTimeoutConstant = 1000 ;
  	if(!SetCommTimeouts(m_hPort, &ctoPort ))
	{
		errorMsg(m_bPort,"SetCommTimeouts");
		ClosePort();
		return;
	}
   //
   ////////////////////////////////////////////////////////////////////
		
   /////////////////////////////////////////////////////////////////
   // SetPortParams
	if(!SetPortParams())
	{
		return;
	}
   //
   ////////////////////////////////////////////////////////////////////
   m_fConnect = TRUE;
} 
/////////////////////////////////////////////////////////////////
// FUNCTION : VOID CSerialComm::ClosePort()
// PURPOSE :  
VOID CSerialComm::ClosePort()
{
	if (m_hPort != INVALID_HANDLE_VALUE )
	{
		if(!CloseHandle(m_hPort))
		{
			errorMsg(m_bPort,"CloseHandle");
		}
		else
		{
			m_hPort = INVALID_HANDLE_VALUE ;
		}
	}
	m_fConnect = FALSE;
}
/////////////////////////////////////////////////////////////////
// FUNCTION : 	BOOL CSerialComm::SetPortParams(LPSTR lpszParams);
// PURPOSE :    , 
BOOL CSerialComm::SetPortParams()
{
   DCB        dcb ;
   
   dcb.DCBlength = sizeof( DCB ) ;
   if(!GetCommState( m_hPort, &dcb ))
   {
	   errorMsg(m_bPort,"GetCommState");
	   ClosePort();
	   return FALSE;
   }
   dcb.BaudRate = m_dwBaudRate;
   dcb.ByteSize = m_bByteSize;
   dcb.Parity = m_bParity;
   dcb.StopBits = m_bStopBits;
   
   /*dcb.fBinary = TRUE;
   dcb.fParity = TRUE;
   dcb.fOutxDsrFlow = TRUE ;
   dcb.fDtrControl = DTR_CONTROL_ENABLE;//DTR_CONTROL_HANDSHAKE;// ;DTR_CONTROL_DISABLE//DTR_CONTROL_ENABLE;//
   dcb.fOutxCtsFlow = FALSE ;
   dcb.fRtsControl = RTS_CONTROL_DISABLE;
   dcb.fInX = dcb.fOutX = FALSE ;
   */
   if(!SetCommState( m_hPort, &dcb ))
   {
	   errorMsg(m_bPort,"SetCommState");
	   ClosePort();
	   return FALSE;
   }
   return TRUE;
} 
/////////////////////////////////////////////////////////////////
// FUNCTION : 	VOID CSerialComm::SetPortNumber(BYTE bNumber)
// PURPOSE : set serial comm number 
// PAPAMETERS : 1 - COM1, 2 - COM2 ...
VOID CSerialComm::SetPortNumber(BYTE bNumber)
{	
	if(!m_fConnect)
	{
		m_bPort = bNumber;
	}
}
/////////////////////////////////////////////////////////////////
// FUNCTION : BYTE CSerialComm::GetPortNumber() const 	
// PURPOSE : return serial comm number 
BYTE CSerialComm::GetPortNumber() const 
{
	return m_bPort;
}
/////////////////////////////////////////////////////////////////
// FUNCTION :VOID CSerialComm::SetBaudRate(DWORD dwBaudRateA)
// PURPOSE : Set BaudRate 
// PAPAMETERS : dwBaudRateA = (...,CBR_9600 ,... )
VOID CSerialComm::SetBaudRate(DWORD dwBaudRateA)
{
   m_dwBaudRate = dwBaudRateA;
   if(m_fConnect)
   {
	   SetPortParams();
   }
}
/////////////////////////////////////////////////////////////////
// FUNCTION : DWORD CSerialComm::GetBaudRate() const	
// PURPOSE : return  BaudRate
DWORD CSerialComm::GetBaudRate() const
{
	return  m_dwBaudRate;;
}
/////////////////////////////////////////////////////////////////
// FUNCTION : VOID CSerialComm::SetByteSize(BYTE bByteSizeA)	
// PURPOSE : Set ByteSize 
// PAPAMETERS : BYTE bByteSizeA = (4,...,8)
VOID CSerialComm::SetByteSize(BYTE bByteSizeA)
{
	m_bByteSize =	bByteSizeA;
	if(m_fConnect)
   {
	   SetPortParams();
   }
}
/////////////////////////////////////////////////////////////////
// FUNCTION : BYTE CSerialComm::GetByteSize() const	
// PURPOSE : Get ByteSize
BYTE CSerialComm::GetByteSize() const
{
	return m_bByteSize;
}
/////////////////////////////////////////////////////////////////
// FUNCTION : 	VOID CSerialComm::SetStopBits(BYTE bStopBitsA)
// PURPOSE :  Set StopBits 
// PAPAMETERS : BYTE bStopBitsA = 	(...,TWOSTOPBITS,...)
VOID CSerialComm::SetStopBits(BYTE bStopBitsA)
{
	m_bStopBits = bStopBitsA; 
	if(m_fConnect)
    {
	   SetPortParams();
    }
}
/////////////////////////////////////////////////////////////////
// FUNCTION : BYTE CSerialComm::GetStopBits() const	
// PURPOSE : return  StopBits
BYTE CSerialComm::GetStopBits() const
{
	return m_bStopBits;
}
/////////////////////////////////////////////////////////////////
// FUNCTION : 	VOID CSerialComm::SetParity(BYTE bParityA)
// PURPOSE :  Set Parity
// PAPAMETERS : bParityA = (...,NOPARITY,..)	
VOID CSerialComm::SetParity(BYTE bParityA)
{
	m_bParity = bParityA;
	if(m_fConnect)
   {
	   SetPortParams();
   }
}
/////////////////////////////////////////////////////////////////
// FUNCTION : BYTE CSerialComm::GetParity() const	
// PURPOSE : return Parity
BYTE CSerialComm::GetParity() const
{
	return m_bParity;
}
//////////////////////////////////////////////////////////////////
//  FUNCTION: INT ReadBlock( LPBYTE lpByte, DWORD dwLength )
//  PURPOSE: read frrom potr to lpByte
INT CSerialComm::ReadBlock( LPBYTE lpByte, DWORD dwLength )
{
	DWORD      dwErrorFlags;
	COMSTAT    ComStat;
	
	if(m_fConnect)
    {
		if (!ReadFile(m_hPort ,lpByte,dwLength,&dwLength,NULL))
		{
			errorMsg(m_bPort,"Read COM");
			ClearCommError(m_hPort, &dwErrorFlags, &ComStat ) ;
			return -1;  
		}
		return ( dwLength ) ;
	}
	return -1;  
}
//////////////////////////////////////////////////////////////////
//  FUNCTION: INT WriteBlock(  LPBYTE lpByte , DWORD dwBytesToWrite)
//  PURPOSE: write to potr from lpByte
INT CSerialComm::WriteBlock(  LPBYTE lpByte , DWORD dwBytesToWrite)
{
   DWORD       dwBytesWritten ;
   DWORD       dwErrorFlags;
   COMSTAT    ComStat;
 
   if(m_fConnect)
   {
	   if (!WriteFile(m_hPort , lpByte, dwBytesToWrite,&dwBytesWritten, NULL))
	   {
			errorMsg(m_bPort,"Write COM");
			ClearCommError(m_hPort , &dwErrorFlags,&ComStat) ;
			return ( -1 );
	   }
	   return ( dwBytesWritten ) ;
   }
   return ( -1 );

}
/////////////////////////////////////////////////////////////////
// FUNCTION : 	VOID CSerialComm::SetDTR();
// PURPOSE :  Set DTR signal
VOID CSerialComm::SetDTR()
{
	if(m_fConnect)
    {
		if(!EscapeCommFunction(m_hPort,SETDTR))
		{
			errorMsg(m_bPort,"EscapeCommFunction");
		}
	}
}
/////////////////////////////////////////////////////////////////
// FUNCTION : VOID CSerialComm::DropDTR();
// PURPOSE :  drop DTR signal
VOID CSerialComm::DropDTR()
{
	if(m_fConnect)
    {
		if(!EscapeCommFunction(m_hPort,CLRDTR))
		{
			errorMsg(m_bPort,"EscapeCommFunction");
		}
	}
}
/////////////////////////////////////////////////////////////////
// FUNCTION : VOID CSerialComm::SendBreak()
// PURPOSE :  send BREAK
VOID CSerialComm::SendBreak()
{
	if(m_fConnect)
    {
		if(!SetCommBreak(m_hPort))
		{
		errorMsg(m_bPort,"SetCommBreak");
		}
		Sleep(BREAK_INTERVAL);
		if(!ClearCommBreak(m_hPort))
		{
			errorMsg(m_bPort,"ClearCommBreak");
		}
	}
}
/////////////////////////////////////////////////////////////////
// FUNCTION : VOID CSerialComm::SendByte( BYTE bByte)
// PURPOSE :  send bByte in COM
VOID CSerialComm::SendByte( BYTE bByte)
{
	WriteBlock(&bByte, sizeof(BYTE));
}
/////////////////////////////////////////////////////////////////
// FUNCTION : BYTE CSerialComm::ReceiveByte()
// PURPOSE :  receive byte from COM
BYTE CSerialComm::ReceiveByte()
{
	BYTE bByte = 0;
	
	ReadBlock(&bByte, sizeof(BYTE));
	
	return bByte;
}
/////////////////////////////////////////////////////////////////
// FUNCTION : VOID ClearPort()
// PURPOSE :  clear COM
VOID CSerialComm::ClearPort()
{
   	if(m_fConnect)
    {
		if(!PurgeComm(m_hPort,PURGE_TXABORT|PURGE_RXABORT|PURGE_TXCLEAR|PURGE_RXCLEAR))
		{
			errorMsg(m_bPort,"PurgeComm");
			ClosePort();
		}
	}
}
/////////////////////////////////////////////////////////////////
// FUNCTION : BOOL CSerialComm::Connected() const
// RETURN : TRUE  - port open
//          FALSE  - port close
BOOL CSerialComm::Connected() const
{
	return m_fConnect;
}

void CSerialComm::SetReadTimeOuts(DWORD dwReadTime)
{
	COMMTIMEOUTS ctoPort;
	
	if(m_fConnect)
	{
		if(!GetCommTimeouts(m_hPort,&ctoPort))
		{
			errorMsg(m_bPort,"SetCommTimeouts");
			ClosePort();
			return;
		}
		ctoPort.ReadTotalTimeoutConstant = dwReadTime;
		
		if(!SetCommTimeouts(m_hPort, &ctoPort ))
		{
			errorMsg(m_bPort,"SetCommTimeouts");
			ClosePort();
			return;
		}
	
	}
	
}

void CSerialComm::Flush()
{
	if(m_fConnect)
    {		
  		if(!FlushFileBuffers(m_hPort))
		{
			errorMsg(m_bPort,"FlushFileBuffers");
			ClosePort();
		}
	}

}
