///////////////////////////////////////////////////////////////
//  other used functions
//
// FILE: funclib.cpp
//
// (C) Borisov E.S. , Kiev UA
//
// CREATED:07.09.1999
///////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "funclib.h"

LPTSTR GetLastErrorText( LPSTR lpszBuf, DWORD dwSize );

//////////////////////////////////////////////////////////////////
//  FUNCTION: VOID errorMsg(LPTSTR lpszMsg)
//  PURPOSE: Print error message 
//  PARAMETERS: lpszMsg - text message
void errorMsg(BYTE bPort,LPSTR lpszMsgA)
{
	
	LPSTR  lpszMsg = new CHAR[500];
	LPSTR  lpszErr = new CHAR[255];

	sprintf(lpszMsg,"COM%d : %s - %s",bPort,lpszMsgA,GetLastErrorText(lpszErr,255));
		
	fprintf(stdout,"%s\n",lpszMsg);
    //MessageBox(NULL,GetLastErrorText(lpszErr,255),lpszMsg,MB_OK|MB_ICONERROR);

	delete [] lpszMsg;
	delete [] lpszErr;
}

////////////////////////////////////////////////////////////
// FUNCTION : BYTE ASCIItoISO(BYTE bASCII)
// PURPOSE :      ASCII   ISO
// PARAMETERS : BYTE bASCII -     ASCII
// RETURN : BYTE  -     ISO  
BYTE ASCIItoISO(BYTE bASCII)
{
	BYTE b = '\x80',
		 c,
		 d,
		 f = bASCII,
		 i,
		 sign=0;
	
	
	for(i=0;i<7;i++)
	  {
	    c = bASCII>>1;
	    d = c<<1;
	    if(bASCII^d)
		{
			sign = !sign;
		}
	    bASCII = c;
	  }
	if(sign)f |= b;
	return(f);
}
////////////////////////////////////////////////////////////
// FUNCTION : BYTE ISOtoASCII(BYTE bISO)
// PURPOSE :       ISO   ASCII
// PARAMETERS : BYTE  bISO -      bISO
// RETURN : BYTE  -     ASCII  
BYTE ISOtoASCII(BYTE bISO)
{
	return(bISO & 0x7F);
}



//////////////////////////////////////////////////////////////////
//  FUNCTION: BOOL GetReceiveParams(...)
//  PURPOSE:  read params from registry
//  PARAMETERS: BYTE bNumbertool  - tool number (in)
//			    LPSTR lpszDir	  - CP dir (out)
//			    DWORD dwSizeDirParam - size of lpszDir buffer (in)
//			    LPSTR lpszFileType - CP type (out)
//			    DWORD dwSizeFileParam -size of lpszFileType buffer (in)
// RETURN : TRUE -success 
BOOL GetReceiveParams( BYTE bNumbertool,
				       LPSTR lpszDir,
				       DWORD dwSizeDirParam,
				       LPSTR lpszFileType,
				       DWORD dwSizeFileParam )
{
	LONG lRc;
	HKEY hReadKey;
	CHAR szRegPath[254] = "";
	DWORD dwDataType = 0;

	sprintf(szRegPath,"SOFTWARE\\ANTK\\Tools_server\\Tools\\%d",bNumbertool);	
	
	lRc = RegOpenKeyEx(  HKEY_LOCAL_MACHINE,
						 szRegPath,
						 0,   
						 KEY_READ,
						 &hReadKey  );
	if(lRc != ERROR_SUCCESS)
	{
		return FALSE;
	}
	lRc = RegQueryValueEx(  hReadKey,
							"ReceiveDir",
							NULL,
							&dwDataType,
							(LPBYTE)lpszDir,
							&dwSizeDirParam);
	if(lRc != ERROR_SUCCESS)
	{
		lRc = RegCloseKey(hReadKey);
		return FALSE;
	}
	
		
	lRc = RegQueryValueEx(  hReadKey,
							"ReceiveFileExt",
							NULL,
							&dwDataType,
							(LPBYTE)lpszFileType,
							&dwSizeFileParam);
		
	if(lRc != ERROR_SUCCESS)
	{
		lRc = RegCloseKey(hReadKey);
		return FALSE;
	}
	
	lRc = RegCloseKey(hReadKey);
	return TRUE;
}
//////////////////////////////////////////////////////////////////
//  FUNCTION: BOOL GetSendParams(...)
//  PURPOSE:  read params from registry
//  PARAMETERS: BYTE bNumbertool  - tool number (in)
//			    LPSTR lpszDir	  - CP dir (out)
//			    DWORD dwSizeDirParam - size of lpszDir buffer (in)
//			    LPSTR lpszFileType - CP type (out)
//			    DWORD dwSizeFileParam -size of lpszFileType buffer (in)
// RETURN : TRUE -success 
BOOL GetSendParams( BYTE bNumbertool,
				   LPSTR lpszDir,
				   DWORD dwSizeDirParam,
				   LPSTR lpszFileType,
				   DWORD dwSizeFileParam )
{
	LONG lRc;
	HKEY hReadKey;
	CHAR szRegPath[254] = "";
	DWORD dwDataType = 0;

	sprintf(szRegPath,"SOFTWARE\\ANTK\\Tools_server\\Tools\\%d",bNumbertool);	
	
	lRc = RegOpenKeyEx(  HKEY_LOCAL_MACHINE,
						 szRegPath,
						 0,   
						 KEY_READ,
						 &hReadKey  );
	if(lRc != ERROR_SUCCESS)
	{
		return FALSE;
	}
	lRc = RegQueryValueEx(  hReadKey,
							"CPDir",
							NULL,
							&dwDataType,
							(LPBYTE)lpszDir,
							&dwSizeDirParam);
	if(lRc != ERROR_SUCCESS)
	{
		lRc = RegCloseKey(hReadKey);
		return FALSE;
	}
	
		
	lRc = RegQueryValueEx(  hReadKey,
							"FileType",
							NULL,
							&dwDataType,
							(LPBYTE)lpszFileType,
							&dwSizeFileParam);
		
	if(lRc != ERROR_SUCCESS)
	{
		lRc = RegCloseKey(hReadKey);
		return FALSE;
	}
	
	lRc = RegCloseKey(hReadKey);
	return TRUE;
}
//////////////////////////////////////////////////////////////////
//  FUNCTION: GetLastErrorText
//  PURPOSE: copies error message text to string
//  PARAMETERS: lpszBuf - destination buffer   dwSize - size of buffer
//  RETURN VALUE:  destination buffer
LPTSTR 
GetLastErrorText( LPSTR lpszBuf, DWORD dwSize )
{
    DWORD dwRet;
    LPTSTR lpszTemp = NULL;

    dwRet = FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		                   FORMAT_MESSAGE_FROM_SYSTEM |
						   FORMAT_MESSAGE_ARGUMENT_ARRAY,
                           NULL,
                           GetLastError(),
                           LANG_NEUTRAL,
                           (LPTSTR)&lpszTemp,
                           0,
                           NULL );
    // supplied buffer is not long enough
    if ( !dwRet || ( (long)dwSize < (long)dwRet+14 ) )
	{
        lpszBuf[0] = '\0';
	}
    else
    {
        lpszTemp[lstrlen(lpszTemp)-2] ='\0'; 
		//remove cr and newline character
        sprintf( lpszBuf,"%s (0x%x)", lpszTemp, GetLastError());
    }
    if ( lpszTemp ) LocalFree((HLOCAL) lpszTemp );

    return lpszBuf;
}
